#!/bin/bash

SEM_CWD=$PWD

SEM_INSTDIR=$HOME/.sonicwall/EndPointManager/bin
SEM_LIBDIR=$SEM_INSTDIR/../lib

# Space-delimited paths that we examine for dependencies
SEM_LIBPATHS="/usr/lib /lib"

# Lib path for 64-bit machines
SEM_LIBPATHS_32="/usr/lib"

if [ -d "/usr/lib32" ]; then
    SEM_LIBPATHS_32="/usr/lib32"
fi

SEM_PRODUCT="Secure Endpoint Manager"
SEM_BIN=SEM
SEM_STDCPLUS=libstdc++.*
export LD_LIBRARY_PATH=$SEM_INSTDIR

function sem_irresolvable_dependencies {
    echo "One or more dependencies cannot be resolved."
    echo "$SEM_PRODUCT install failed."
    echo
    echo "Error details:"
    ldd $SEM_INSTDIR/$SEM_BIN
    return 1
}

function sem_resolve_in_path {
    basef=$1
    wanted=$2
    libpath=$3

    if [ ! -d "$libpath" ]; then
        # We can't meaningfully report the problem without spamming the console
        # since this function can be called many times.
        return 1
    fi

    if [ -f "$libpath/$wanted" ]; then
        return 0
    fi
   
    if [ -f "$libpath/$basef" ]; then
        echo "Linking $wanted to $libpath/$basef"
        ln -sf "$libpath/$basef" "$SEM_INSTDIR/$wanted"
        return 0
    fi

    # We need to grub around a bit. Ick
    for rfile in `ls $libpath/$basef.* 2>/dev/null`; do
        if [ -f "$rfile" ]; then
            echo "Linking $wanted to $rfile"
            ln -sf "$rfile" "$SEM_INSTDIR/$wanted"
            return 0
        fi
    done

    return 1
}

function sem_resolve_dependency {
    basef=$1
    wanted=$2

    # In theory at least, bash doesn't spawn subshells for for-loops, but there
    # are rumours that that is not a firm guarantee, in which case the return 0
    # will not do what we want it to do.
    
    MY_SEM_LIBPATHS=$SEM_LIBPATHS
    XG_ARCH=`uname -m`
    
    if [ "$XG_ARCH" == "x86_64" -o "$XG_ARCH" == "X86_64" ]; then

	MY_SEM_LIBPATHS=$SEM_LIBPATHS_32
    fi
		
    for libdir in $MY_SEM_LIBPATHS; do
        sem_resolve_in_path "$basef" "$wanted" "$libdir" && return 0
    done

    sem_irresolvable_dependencies
    return 1
}

function sem_fix_dependencies {
    ldd $SEM_INSTDIR/$SEM_BIN 2>&1 | grep 'not found' | awk '{print $1}' \
            | while read brokefile
    do
        basefile=`echo $brokefile | sed -e 's/\.so.*$/.so/'`
        if [ "$basefile" == "libcrypto.so" ]; then
            sem_resolve_dependency $basefile $brokefile || return 1
        else
            sem_irresolvable_dependencies
            return 1
        fi
    done
    return $?
}

echo "Installing $SEM_PRODUCT using ia32-lib[$SEM_LIBPATHS_32]..."

mkdir -p $SEM_INSTDIR
[ -f "$SEM_CWD/$SEM_BIN" ] && mv "$SEM_CWD/$SEM_BIN" "$SEM_INSTDIR"
[ -f "$SEM_CWD/cert.pem" ] && mv "$SEM_CWD/cert.pem" "$SEM_INSTDIR"
[ -f "$SEM_CWD/version" ] && mv "$SEM_CWD/version" "$SEM_INSTDIR"

mkdir -p $SEM_LIBDIR
mv $SEM_CWD/$SEM_STDCPLUS $SEM_LIBDIR

# Change permissions for SEM module

# AvConnect and SEM are setuid. Maybe we should ask the user if this is
# what they want?
chown -R root:root "$SEM_INSTDIR/$SEM_BIN"
chmod 4755 "$SEM_INSTDIR/$SEM_BIN"

# Repeated dependency checker, so that we bail out if one of our symlinks
# went rogue
sem_fix_dependencies || exit 1
sem_fix_dependencies || exit 1

# And one last direct check, since sem_fix_dependencies is prone to false cheer
UNFOUND=`ldd $SEM_INSTDIR/$SEM_BIN 2>&1 | grep 'not found'`
if [ "$UNFOUND" != "" ]; then
    echo "Aborting install, $SEM_BIN has unresolved dependencies"
    ldd "$SEM_INSTDIR/$SEM_BIN"
    exit 1
fi

echo "$SEM_PRODUCT install finished."
exit 0

